"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zhin_1 = require("zhin");
const web_wechat_1 = require("web-wechat");
const utils_1 = require("./utils");
const wechatAdapter = new zhin_1.Adapter('wechat');
wechatAdapter.define('sendMsg', async (bot_id, target_id, target_type, message, source) => {
    const bot = wechatAdapter.pick(bot_id);
    let msg = await wechatAdapter.app.renderMessage(message, source);
    msg = (0, utils_1.formatSendable)(msg);
    switch (target_type) {
        case 'group':
            return bot.sendGroupMsg(target_id, msg);
        case 'private':
            return bot.sendPrivateMsg(target_id, msg);
        default:
            throw new Error(`wechat适配器暂不支持发送${target_type}类型的消息`);
    }
});
const startBots = (configs) => {
    for (const config of configs) {
        const bot = new web_wechat_1.Client(config);
        Object.defineProperties(bot, {
            unique_id: {
                value: config.unique_id,
                writable: false,
            },
            quote_self: {
                get() {
                    return wechatAdapter.botConfig(bot)?.command_prefix;
                },
            },
            forward_length: {
                get() {
                    return wechatAdapter.botConfig(bot)?.forward_length;
                },
            },
            command_prefix: {
                get() {
                    return wechatAdapter.botConfig(bot)?.command_prefix;
                },
            },
        });
        bot.on('message', messageHandler.bind(global, bot));
        bot.start().then(() => {
            wechatAdapter.emit('bot-ready', bot);
        });
        wechatAdapter.bots.push(bot);
    }
};
const messageHandler = (bot, event) => {
    const message = zhin_1.Message.fromEvent(wechatAdapter, bot, event);
    message.raw_message = (0, utils_1.sendableToString)(event.message).trim();
    message.from_id = event instanceof web_wechat_1.PrivateMessageEvent ? event.user_id : event.group_id;
    message.message_type = event.message_type;
    const master = wechatAdapter.botConfig(bot)?.master;
    const admins = wechatAdapter.botConfig(bot)?.admins;
    message.sender = {
        ...event.sender,
        permissions: [
            master && event.sender.user_id === master && 'master',
            admins && admins.includes(event.sender.user_id) && 'admins',
        ].filter(Boolean),
    };
    wechatAdapter.app.emit('message', wechatAdapter, bot, message);
};
const stopBots = () => {
    for (const bot of wechatAdapter.bots) {
        bot.stop();
    }
};
wechatAdapter.on('start', startBots);
wechatAdapter.on('stop', stopBots);
exports.default = wechatAdapter;
